package cn.cloudwalk.smartframework.transportcomponents.exchange.support.header;

import cn.cloudwalk.smartframework.transportcomponents.Channel;
import cn.cloudwalk.smartframework.transportcomponents.Client;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.Collection;

/**
 * 心跳处理任务
 *
 * 备注：只有服务端使用了该线程处理客户端超时未发送数据的情况 客户端心跳由客户端自己处理
 *
 * @author LIYANHUI
 * @since 1.0.0
 */
final class HeartBeatTask implements Runnable {

    private static final Logger logger = LogManager.getLogger(HeartBeatTask.class);

    private ChannelProvider channelProvider;

    private int heartbeat;

    private int heartbeatTimeout;

    HeartBeatTask(ChannelProvider provider, int heartbeat, int heartbeatTimeout) {
        this.channelProvider = provider;
        this.heartbeat = heartbeat;
        this.heartbeatTimeout = heartbeatTimeout;
    }

    @Override
    public void run() {
        try {
            long now = System.currentTimeMillis();
            for (Channel channel : channelProvider.getChannels()) {
                if (channel.isClosed()) {
                    continue;
                }
                try {
                    Long lastRead = (Long) channel.getAttribute(
                            HeaderExchangeHandler.KEY_READ_TIMESTAMP);
                    Long lastWrite = (Long) channel.getAttribute(
                            HeaderExchangeHandler.KEY_WRITE_TIMESTAMP);
                    if ((lastRead != null && now - lastRead > heartbeat)
                            || (lastWrite != null && now - lastWrite > heartbeat)) {
                        //发送心跳数据
                    }
                    if (lastRead != null && now - lastRead > heartbeatTimeout) {
                        logger.warn("close channel：" + channel
                                + ", because no data was received for a long time, timeout time： " + heartbeatTimeout + "ms");
                        if (channel instanceof Client) {
                            try {
                                ((Client) channel).reconnect();
                            } catch (Exception e) {
                            }
                        } else {
                            channel.close();
                        }
                    }
                } catch (Throwable t) {
                    logger.warn("Heartbeat thread exception ，" + channel.getRemoteAddress(), t);
                }
            }
        } catch (Throwable t) {
            logger.warn("Uncontrollable exceptions in heartbeat threads : " + t.getMessage(), t);
        }
    }

    interface ChannelProvider {
        Collection<Channel> getChannels();
    }

}

